package net.common;

import java.util.List;
import java.util.Iterator;

import net.common.model.Auto;
import net.common.model.User;

import org.hibernate.Criteria;
import org.hibernate.Query;
// import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.criterion.Restrictions;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.boot.registry.StandardServiceRegistryBuilder;

public class HibernateExample
{
	private  Session  session = null;
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Процедура создания сессии
	 * @return org.hibernate.Session
	 */
	private Session createHibernateSession()
	{
		SessionFactory   sf  = null;
		ServiceRegistry  srvc = null;
		try {
			Configuration conf = new Configuration();
			conf.configure("hibernate.cfg.xml");
					
			srvc = new StandardServiceRegistryBuilder().applySettings(
                                              conf.getProperties()).build();
			sf = conf.buildSessionFactory(srvc);
			session = sf.openSession();
			System.out.println("Создание сессии");
		} catch (Throwable e) {
			System.err.println("Failed to create sessionFactory object : " + e.getMessage());
				throw new ExceptionInInitializerError(e);
		}
		return session;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	protected void saveUser()
	{
		if (session == null)
			return;
		
		User user1 = new User();
		user1.setName ("Иван");
		user1.setLogin("ivan");
		
		User user2 = new User();
		user2.setName ("Сергей");
		user2.setLogin("serg"  );

		Transaction trans = session.beginTransaction();
		session.save(user1);
		session.save(user2);
		
		Auto auto = new Auto();
		auto.setName("Volvo");
		auto.setUser(user1);
		session.saveOrUpdate(auto);
		
		auto = new Auto();
		auto.setName("Skoda");
		auto.setUser(user1);
		session.saveOrUpdate(auto);
		
		session.flush();
        trans.commit();
        // Обновление detached-объект (выполнение select к БД) и преобразование его в persistent 
        session.refresh(user1);

//      System.out.println("user1 : " + user1.toString());
//		System.out.println("user2 : " + user2.toString());
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void readUsers_HQL()
	{
		System.out.println("\n\nЧтение записей : HQL");
		// Использование HQL (Hibernate Query Language) 
		String sql = "From " + User.class.getSimpleName();
		@SuppressWarnings("unchecked")
		List<User> users = (List<User>)session.createQuery(sql).list();

		System.out.println("users.size = " + users.size());
		for (Iterator<User> it = users.iterator(); it.hasNext();){
			User user = (User) it.next();
			System.out.println(user.toString());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void readUsers_SQL()
	{
		System.out.println("\n\nЧтение записей : SQL");
		// Использование native SQL query
		String sql = "select ID, LOGIN, NAME from USERS";
		Query query = session.createSQLQuery(sql).addEntity(User.class);
		@SuppressWarnings("unchecked")
		List<User> users = query.list();
		
		System.out.println("users.size = " + users.size());
		for (Iterator<User> it = users.iterator(); it.hasNext();){
			User user = (User) it.next();
			System.out.println(user.toString());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void readObjects()
	{
		System.out.println("\n\nЧтение объектов");

		String sql = "select ID, LOGIN, NAME from USERS";
		Query query = session.createSQLQuery(sql);
		@SuppressWarnings("unchecked")
		List<Object[]> rows = query.list();

		for(Object[] row : rows) {
			User user = new User();
			user.setId     (Integer.valueOf(row[0].toString()));
			user.setLogin                  (row[1].toString());
			user.setName                   (row[2].toString());
		    System.out.println(user.toString());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Чтение фильтрованных записей
	 */
	private void readUsers(final String filter)
	{
		// Использование native SQL query с условием
		String sql = "select ID, LOGIN, NAME from USERS where login like :log";
		Query query = session.createSQLQuery(sql)          // SQL
			                 .addEntity(User.class)        // Class
				             .setParameter("log", filter); // Condition
		@SuppressWarnings("unchecked")
		List<User> users = query.list();

		System.out.println("users.size = " + users.size());

		User user = (User) session.load(User.class, 51);
		if (user != null)
			System.out.println("\n\n" + user.toString());
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void readUsers_criteria()
	{
		System.out.println("\n\nCriteria API");
		Criteria criteria = session.createCriteria(User.class)
                                   .add(Restrictions.eq("name", "Иван"));
		@SuppressWarnings("unchecked")
		List<User> users = criteria.list();

		for (Iterator<User> it = users.iterator(); it.hasNext();){
			User user = (User) it.next();
			System.out.println(user.toString());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Конструктор класса
	 */
	public HibernateExample()
	{
		// Создание сессии
		session = createHibernateSession();
		if (session != null) {
        	System.out.println("session is created");
			// Добавление записей в таблицу
//        	saveUser();
        	// Чтение записей таблицы HQL
    		readUsers_HQL();
        	// Чтение записей таблицы SQL
    		readUsers_SQL();
        	// Чтение набора объектов
    		readObjects();
            // Поиск записи по идентификатору
        	readUsers_criteria();
    		
    		// Поиск записи по идентификатору
    		System.out.println("\n\nЧтение записей с фильтрацией");
        	readUsers("ivan");
        	
            // Закрытие сессии
            if (session.isOpen())
                session.close();
        } else {
        	System.out.println("session is not created");
        }
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public static void main(String[] args)
	{
		new HibernateExample();
		System.exit(0);
	}
}